BC_ChainCategoryDB = { head = {}, tail = {} };
BC_ChainActionDB = { head = {}, tail = {} };
BC_ChainCryListDB = { head = {}, tail = {} };


--===================================================================================================================--

-- Chain Functions

--===================================================================================================================--
function BC_ChainName_Get()
	action1 = BC_Selected.chainAction["Head"];
	action2 = BC_Selected.chainAction["Tail"];
	cry1 = BC_Selected.chainCry["Head"];
	cry2 = BC_Selected.chainCry["Tail"];
	if(action1 * action2 * cry1 * cry2 == 0 ) then
		return nil;
	end
	return BC_ChainActionDB["Head"][action1] .. "[" .. BC_ChainCryListDB["Head"][cry1] .. "]: " .. BC_ChainActionDB["Tail"][action2] .. "[" .. BC_ChainCryListDB["Tail"][cry2] .. "]";
end

function BC_ChainHelpTitle_Get()
	action1 = BC_Selected.chainAction["Head"];
	action2 = BC_Selected.chainAction["Tail"];
	cry1 = BC_Selected.chainCry["Head"];
	cry2 = BC_Selected.chainCry["Tail"];
	if(action1 * cry1 > 0) then
		actionName1 = BC_ChainActionDB["Head"][action1];
		cryName1 = BC_ChainCryListDB["Head"][cry1];
		message1 = actionName1 .. "[" .. cryName1 .. "]";
	else
		message1 = "None selected";
	end
	if(action2 * cry2 > 0) then
		actionName2 = BC_ChainActionDB["Tail"][action2];
		cryName2 = BC_ChainCryListDB["Tail"][cry2];
		message2 = actionName2 .. "[" .. cryName2 .. "]";
	else
		message2 = "None selected";
	end
	return message1 .. "\n --> " .. message2;
end

function BC_ChainHelpText_Get()
	action1 = BC_Selected.chainAction["Head"];
	action2 = BC_Selected.chainAction["Tail"];
	cry1 = BC_Selected.chainCry["Head"];
	cry2 = BC_Selected.chainCry["Tail"];
	if(action1 * action2 * cry1 * cry2 == 0 ) then
		return "Selection of cry chain is not yet complete.";
	end
	actionName1 = BC_ChainActionDB["Head"][action1];
	actionName2 = BC_ChainActionDB["Tail"][action2];
	cryName1 = BC_ChainCryListDB["Head"][cry1];
	cryName2 = BC_ChainCryListDB["Tail"][cry2];
	return "When " .. actionName2 .. " is executed immediately after " .. actionName1 .. " and " .. actionName1 .. " executed cry " .. cryName1 .. ", then " .. actionName2 .. " will automatically execute cry " .. cryName2 .. ".";
end

function BC_IsChainStart(actionName, cryNum)
	for i=1, table.getn(BC_Saved.chains) do
		if(BC_Saved.chains[actionName1] == actionName and BC_Saved.chains[cry1] == cryNum) then
			return true;
		end
	end
	return false;
end

--===================================================================================================================--

-- OnLoad/OnShow Functions

--===================================================================================================================--

function BC_EditChainsFrame_OnLoad()
	BC_ChainFrame_SetState(0);
	BC_EditChainsFrame_HelpTitle:Show();
	BC_EditChainsFrame_HelpTitle:SetJustifyH("LEFT");
	BC_EditChainsFrame_HelpTitle:SetJustifyV("TOP");
	BC_EditChainsFrame_HelpText:Show();
	BC_EditChainsFrame_HelpText:SetJustifyH("LEFT");
	BC_EditChainsFrame_HelpText:SetJustifyV("TOP");
end

function BC_SavedChainsFrame_OnLoad()
	for i=1, 5 do
		button = getglobal("SavedChainsFrame_ListItem" .. i);
		button:SetWidth(300);
	end
	SavedChainsFrame:Show();
end

function BC_EditChainsFrame_OnShow()
	BC_ChainFrame_Update();
end

--===================================================================================================================--

-- Frame State/Update Functions

--===================================================================================================================--
function BC_ChainFrame_Update()
	title = BC_ChainHelpTitle_Get();
	message = BC_ChainHelpText_Get();
	BC_EditChainsFrame_HelpTitle:SetText(title);
	BC_EditChainsFrame_HelpText:SetText(message);
	if(BC_Selected.chainCry["Head"] * BC_Selected.chainCry["Tail"] > 0) then
		BC_EditChainsFrame_SaveButton:Enable();
	else
		BC_EditChainsFrame_SaveButton:Disable();
	end
	BC_ChainEditSavedFrame_UpdateList(0, 5);
end


function BC_ChainFrame_SetState(state)
	if(state==0) then
		BC_EditChainsFrame:Hide();
	elseif(state==1) then
		BC_EditChainsFrame:Show();
		BC_SavedChains_SetState(0);
		ChainSelectHead_TitleText:SetText(BC_FRAMELABEL_CHAINHEAD);
		ChainSelectTail_TitleText:SetText(BC_FRAMELABEL_CHAINTAIL);
		SavedChainsFrame_TitleText:SetText(BC_FRAMELABEL_CHAINS);
		BC_ChainEditCategoryFrame_Populate("Head");
		BC_ChainEditCategoryFrame_Populate("Tail");
		BC_SavedChainsFrame_SetState(1);
	end
end

function BC_SavedChains_SetState(state)
	if(state==0) then
		BC_EditChainsFrame_DeleteButton:Hide();
		BC_EditChainsFrame_NewButton:Hide();
		ChainSelectHead:Show();
		ChainSelectTail:Show();
	elseif(state==1) then
		BC_EditChainsFrame_DeleteButton:Show();
		BC_EditChainsFrame_NewButton:Show();
		ChainSelectHead:Hide();
		ChainSelectTail:Hide();
	end
end

function BC_ChainCategoryFrame_SetState(link, state)
	if(state==0) then
		getglobal("ChainSelect" .. link .. "_CategoryFrame"):Hide();
	elseif(state==1) then
		getglobal("ChainSelect" .. link .. "_CategoryFrame"):Show();
		getglobal("ChainSelect" .. link .. "_CategoryFrame_ListHighlight"):Hide();
		getglobal("ChainSelect" .. link .. "_CategoryFrame_ListHighlight"):SetWidth(130);
		getglobal("ChainSelect" .. link .. "_CategoryFrame_ListHighlightTexture"):SetWidth(130);
	end
end

function BC_ChainActionFrame_SetState(link, state)
	if(state==0) then
		getglobal("ChainSelect" .. link .. "_ActionFrame"):Hide();
	elseif(state==1) then
		getglobal("ChainSelect" .. link .. "_ActionFrame"):Show();
		getglobal("ChainSelect" .. link .. "_ActionFrame_ListHighlight"):Hide();
		getglobal("ChainSelect" .. link .. "_ActionFrame_ListHighlight"):SetWidth(180);
		getglobal("ChainSelect" .. link .. "_ActionFrame_ListHighlightTexture"):SetWidth(180);
	end
end

function BC_ChainCryListFrame_SetState(link, state)
	if(state==0) then
		getglobal("ChainSelect" .. link .. "_CryListFrame"):Hide();
	elseif(state==1) then
		getglobal("ChainSelect" .. link .. "_CryListFrame"):Show();
		getglobal("ChainSelect" .. link .. "_CryListFrame_ListHighlight"):Hide();
		getglobal("ChainSelect" .. link .. "_CryListFrame_ListHighlight"):SetWidth(130);
		getglobal("ChainSelect" .. link .. "_CryListFrame_ListHighlightTexture"):SetWidth(130);
	end
end

function BC_SavedChainsFrame_SetState(state)
	if(state==0) then
		getglobal("SavedChainsFrame"):Hide();
	elseif(state==1) then
		getglobal("SavedChainsFrame"):Show();
		getglobal("SavedChainsFrame_ListHighlight"):Hide();
		getglobal("SavedChainsFrame_ListHighlight"):SetWidth(300);
		getglobal("SavedChainsFrame_ListHighlightTexture"):SetWidth(300);
	end
end

--===================================================================================================================--

-- Frame Population Functions

--===================================================================================================================--
function BC_ChainEditCategoryFrame_Populate(link)
	BC_Debug("BC_ChainEditCategoryFrame_Populate(" .. link .. ")", 2);
	BC_ChainCategoryDB[link] = {};
	BC_Selected.chainCategory[link] = 0;
	for key in pairs(BC_Spellbook) do
		tinsert(BC_ChainCategoryDB[link], key);
	end
	sort(BC_ChainCategoryDB[link]);
	BC_ChainEditCategoryFrame_UpdateList(link);
end

function BC_ChainEditActionFrame_Populate(link)
	BC_Debug("BC_ChainEditActionFrame_Populate(" .. link .. ")", 2);
	BC_ChainActionDB[link] = {};
	BC_Selected.chainAction[link] = 0;
	for key in pairs(BC_Spellbook[BC_ChainCategoryDB[link][BC_Selected.chainCategory[link]]]) do
		tinsert(BC_ChainActionDB[link],BC_Spellbook[BC_CategoryDB[BC_Selected.chainCategory[link]]][key]);
	end
	BC_ChainEditActionFrame_UpdateList(link);
end

function BC_ChainEditCryListFrame_Populate(link)
	BC_Debug("BC_ChainEditCryListFrame_Populate(" .. link .. ")", 2);
	BC_ChainCryListDB[link] = {};
	BC_Selected.chainCry[link] = 0;
	if(BC_Saved.cries[BC_ChainActionDB[link][BC_Selected.chainAction[link]]].cry) then
		for i=1,table.getn(BC_Saved.cries[BC_ChainActionDB[link][BC_Selected.chainAction[link]]].cry) do
			tinsert(BC_ChainCryListDB[link],BC_Saved.cries[BC_ChainActionDB[link][BC_Selected.chainAction[link]]].cry[i].name);
		end
	end
	BC_ChainEditCryListFrame_UpdateList(link);
end

--===================================================================================================================--

-- Scroll Functions

--===================================================================================================================--

function BC_SavedChains_UpdateList()
	frame = getglobal("SavedChainsFrame_ListScroll");
	FauxScrollFrame_Update(frame, table.getn(BC_Saved.chains), 5, 16);
	BC_ChainEditSavedFrame_UpdateList(BC_Selected.savedChain, 5);
end

function BC_ChainEditCategoryFrame_UpdateList(link)
	if( link == nil) then
		link = strsub(this:GetName(), 12, 15);
	end
	frame = getglobal("ChainSelect" .. link .. "_CategoryFrame_ListScroll");
	FauxScrollFrame_Update(frame, table.getn(BC_ChainCategoryDB[link]), 5, 16);
	BC_Chains_UpdateList("Category", link, BC_Selected.chainCategory[link], 5);
end

function BC_ChainEditActionFrame_UpdateList(link)
	if( link == nil) then
		link = strsub(this:GetName(),12,15);
	end
	scroll = getglobal("ChainSelect" .. link .. "_ActionFrame_ListScroll");
	FauxScrollFrame_Update(scroll, table.getn(BC_ChainActionDB[link]), 9, 16);
	BC_Chains_UpdateList("Action", link, BC_Selected.chainAction[link], 9);
end

function BC_ChainEditCryListFrame_UpdateList(link)
	if( link == nil) then
		link = strsub(this:GetName(),12,15);
	end
	BC_Debug("BC_ChainEditCryListFrame_UpdateList(" .. link .. ")", 2);
	scroll = getglobal("ChainSelect" .. link .. "_CryListFrame_ListScroll");
	FauxScrollFrame_Update(scroll, table.getn(BC_ChainCryListDB[link]), 9, 16);
	BC_Chains_UpdateList("CryList", link, BC_Selected.chainCry[link], 9);
end

function BC_ChainEditSavedFrame_UpdateList(selected, maximum)
	scrollFrame = getglobal("SavedChainsFrame_ListScroll");
	FauxScrollFrame_Update(scrollFrame, table.getn(BC_Saved.chains), maximum, 16);
	local topItem = FauxScrollFrame_GetOffset(scrollFrame);
	local item=1;
	for i=topItem+1, topItem + maximum do
		local listButton = getglobal("SavedChainsFrame_ListItem" .. item);
		if(BC_Saved.chains[i]) then
			if(listButton) then
				listButton:SetText(BC_Saved.chains[i].name);
				listButton:Show();
			end
		else
			listButton:Hide();
		end
		item = tonumber(item) + 1;
	end
	highlight = selected - topItem;
	highlightButton = getglobal("SavedChainsFrame_ListHighlight");
	if(highlight > 0 and highlight <= maximum) then
		highlightButton:SetPoint("TOPLEFT", "SavedChainsFrame_ListItem" .. tostring(highlight), "TOPLEFT", 5, 0);
		highlightButton:Show();
	else
		highlightButton:Hide();
	end
end

function BC_Chains_UpdateList(frameName, link, selected, maximum)
	scrollFrame = getglobal("ChainSelect" .. link .. "_" .. frameName .. "Frame_ListScroll");
	local topItem = FauxScrollFrame_GetOffset(scrollFrame);
	local item=1;
	for i=topItem+1, topItem + maximum do
		local listButton = getglobal("ChainSelect" .. link .. "_" .. frameName .. "Frame_ListItem" .. item);
		db = getglobal("BC_Chain" .. frameName .. "DB");
		if(db[link][i]) then
			if(listButton) then
				listButton:SetText(db[link][i]);
				listButton:Show();
			end
		else
			listButton:Hide();
		end
		item = tonumber(item) + 1;
	end
	highlight = selected - topItem;
	highlightButton = getglobal("ChainSelect" .. link .. "_" .. frameName .. "Frame_ListHighlight");
	if(highlight > 0 and highlight <= maximum) then
		highlightButton:SetPoint("TOPLEFT", "ChainSelect" .. link .. "_" .. frameName .. "Frame_ListItem"..tostring(highlight), "TOPLEFT", 5, 0);
		highlightButton:Show();
	else
		highlightButton:Hide();
	end
end

--===================================================================================================================--

-- List Button Functions

--===================================================================================================================--

function BC_ChainCategoryButton_OnClick(link, index)
	BC_Debug("BC_ChainCategoryButton_OnClick(" .. link .. ", " .. tostring(index) .. ")", 2);
	if(index > 0) then
		BC_ChainActionFrame_SetState(link, 1);
		BC_ChainCryListFrame_SetState(link, 0);
		scroll = getglobal("ChainSelect" .. link .. "_CategoryFrame_ListScroll");
		BC_Selected.chainCategory[link] = index + FauxScrollFrame_GetOffset(scroll);
		BC_ChainEditActionFrame_Populate(link);
	end
	BC_ChainFrame_Update();
end

function BC_ChainActionButton_OnClick(link, index)
	BC_Debug("BC_ChainActionButton_OnClick(" .. link .. ", " .. tostring(index) .. ")", 2);
	if(index > 0) then
		BC_ChainCryListFrame_SetState(link, 1);
		scroll = getglobal("ChainSelect" .. link .. "_ActionFrame_ListScroll");
		BC_Selected.chainAction[link] = index + FauxScrollFrame_GetOffset(scroll);
		BC_ChainEditCryListFrame_Populate(link);
	end
	BC_ChainFrame_Update();
end

function BC_ChainCryListButton_OnClick(link, index)
	BC_Debug("BC_ChainCryListButton_OnClick(" .. link .. ", " .. tostring(index) .. ")", 2);
	if(index > 0) then
		scroll = getglobal("ChainSelect" .. link .. "_CryListFrame_ListScroll");
		BC_Selected.chainCry[link] = index + FauxScrollFrame_GetOffset(scroll);
	end
	BC_ChainFrame_Update();
end

function BC_SavedChainsListButton_OnClick(index)
	BC_Debug("BC_SavedChainsListButton_OnClick(" .. tostring(index) .. ")", 2);
	if(index > 0) then
		BC_Selected.savedChain = index + FauxScrollFrame_GetOffset(SavedChainsFrame_ListScroll);
		ChainSelectHead:Hide();
		ChainSelectTail:Hide();
		BC_SavedChains_SetState(1);
		actionName1 = BC_Saved.chains[BC_Selected.savedChain].actionName1;
		actionName2 = BC_Saved.chains[BC_Selected.savedChain].actionName2;
		cryName1 = BC_Saved.cries[actionName1].cry[BC_Saved.chains[BC_Selected.savedChain].cry1].name;
		cryName2 = BC_Saved.cries[actionName2].cry[BC_Saved.chains[BC_Selected.savedChain].cry2].name;
		BC_EditChainsFrame_HelpTitle:SetText(actionName1 .. "[" .. cryName1 .. "]\n --> " .. actionName2 .. "[" .. cryName2 .. "]");
		message = BC_TEXT_CHAINEXPLAIN;
		message = gsub(message, "XACTION1X", actionName1);
		message = gsub(message, "XACTION2X", actionName2);
		message = gsub(message, "XCRY1X", cryName1);
		message = gsub(message, "XCRY2X", cryName2);
		BC_EditChainsFrame_HelpText:SetText(message);
	end
end

--===================================================================================================================--

-- Button Functions

--===================================================================================================================--

function BC_EditChainsFrame_SaveButton_OnClick()
	BC_Debug("BC_EditChainsFrame_SaveButton_OnClick()", 2);
	saveName = BC_ChainName_Get();
	if(saveName) then
		if(BC_EditChains_GetExists(BC_ChainActionDB["Head"][BC_Selected.chainAction["Head"]], BC_ChainActionDB["Tail"][BC_Selected.chainAction["Tail"]], BC_Selected.chainCry["Head"], BC_Selected.chainCry["Tail"])) then
			BC_Message(gsub(BC_TEXT_CHAINSAVEFAIL01, "XNAMEX", saveName));
		else
			BC_Message(gsub(BC_TEXT_CHAINSAVESUCCESS, "XNAMEX", saveName));
			tinsert(BC_Saved.chains, {
				name = saveName,
				cat1 = BC_ChainCategoryDB["Head"][BC_Selected.chainCategory["Head"]],
				cat2 = BC_ChainCategoryDB["Tail"][BC_Selected.chainCategory["Tail"]],
				actionName1 = BC_ChainActionDB["Head"][BC_Selected.chainAction["Head"]],
				actionName2 = BC_ChainActionDB["Tail"][BC_Selected.chainAction["Tail"]],
				cry1 = BC_Selected.chainCry["Head"],
				cry2 = BC_Selected.chainCry["Tail"],
				});
		end
		BC_ChainFrame_Update();
	end
end

function BC_EditChainsFrame_NewButton_OnClick()
	BC_Debug("BC_EditChainsFrame_NewButton_OnClick()", 2);
	BC_Selected.savedChain = 0;
	BC_SavedChainsFrame_SetState(1);
	BC_SavedChains_SetState(0);
end

function BC_EditChainsFrame_DeleteButton_OnClick()
	BC_Debug("BC_EditChainsFrame_DeleteButton_OnClick()", 2);
	if(BC_Selected.savedChain > 0) then
		BC_Message(gsub(BC_TEXT_CHAINREMOVE, "XNAMEX", BC_Saved.chains[BC_Selected.savedChain].name));
		tremove(BC_Saved.chains,BC_Selected.savedChain);
		BC_Selected.savedChain = 0;
		BC_ChainEditSavedFrame_UpdateList(0, 5);
	end
end


function BC_EditChains_GetExists(actionName1, actionName2, cry1, cry2)
	for i=1, table.getn(BC_Saved.chains) do
		if(BC_Saved.chains[i].actionName1 == actionName1) then
			if(BC_Saved.chains[i].actionName2 == actionName2) then
				if(BC_Saved.chains[i].cry1 == cry1) then
					if(BC_Saved.chains[i].cry2 == cry2) then
						return true;
					end
				end
			end
		end
	end
	return false;
end