--===================================================================================================================--

--OnLoad Functions

--===================================================================================================================--

function BC_EditConditionsFrame_OnLoad()
	BC_EditConditionsFrame_SetState(0);
	BC_EditConditionFrame_ConditionFrame_Populate();
	BC_EditConditionsFrame_HelpText_SetText(1);
end

function BC_EditConditionsFrame_OnShow()
	BC_Conditions.ready = {};
	for i=1, table.getn(BC_Conditions.start) do
		BC_Conditions.ready[i] = BC_Conditions.start[i];
	end
	BC_EditConditionFrame_ConditionFrame_Populate();
end

function BC_EditConditionFrame_ConditionFrame_Populate()
	BC_Debug("BC_ConditionFrame_Populate()", 2);
	BC_ConditionDB = {};
	if(BC_Conditions.ready) then
		for i=1, table.getn(BC_Conditions.ready) do
			conditionName = BC_Conditions.ready[i].target .. " " .. BC_Conditions.ready[i].variable .. " " .. BC_Conditions.ready[i].test .. " " .. BC_Conditions.ready[i].value;
			tinsert(BC_ConditionDB, conditionName);
		end
	end
	BC_EditConditionsFrame_ConditionFrame_UpdateList();
end

function BC_EditConditionsFrame_LoadFromCry(action, cryNum)
	BC_Debug("BC_EditConditionsFrame_LoadFromCry(" .. tostring(action) .. ", " .. tostring(cryNum) .. ")", 2);
	BC_Conditions.start = {};
	BC_Conditions.ready = {};
	if(BC_Saved.cries[action].cry[cryNum].condition) then
		for i=1, table.getn(BC_Saved.cries[action].cry[cryNum].condition) do
			tinsert(BC_Conditions.start, BC_Saved.cries[action].cry[cryNum].condition[i]);
			BC_Conditions.ready[i] = BC_Conditions.start[i];
		end
	end
	BC_EditConditionsFrame_EditBoxValues_Set({target = "", variable = "", test = "", value = "",});
	BC_Conditions.exclusive = BC_Saved.cries[action].cry[cryNum].exclusive;
	BC_EditConditionsFrame_ExclusiveCheckbox:SetChecked(BC_Saved.cries[action].cry[cryNum].exclusive);
end

function BC_EditConditionsFrame_ClearConditions()
	BC_Debug("BC_EditConditionsFrame_ClearConditions()", 2);
	BC_Conditions.valid = false;
	BC_Conditions.exclusive = 0;
	BC_Conditions.start = {};
	BC_Conditions.ready = {};
	BC_EditConditionsFrame_ExclusiveCheckbox:SetChecked(0);
	BC_EditConditionsFrame_EditBoxValues_Set({target = "", variable = "", test = "", value = "",});
end

function BC_EditConditionsFrame_ConditionFrame_UpdateList()
	FauxScrollFrame_Update(BC_EditConditionsFrame_ConditionFrame_ListScroll, table.getn(BC_ConditionDB), 5, 16);
	BC_Universal_UpdateList("BC_EditConditionsFrame_","Condition",BC_Selected.condition, 5);
	if(BC_Selected.condition == 0) then
		BC_EditConditionsFrame_DeleteButton:Hide();
	end
end

function BC_EditConditionsFrame_ListButton_OnClick(index)
	BC_EditConditionsFrame_EditBoxValues_Set(BC_Conditions.ready[index]);
	BC_Selected.condition = index + FauxScrollFrame_GetOffset(BC_EditConditionsFrame_ConditionFrame_ListScroll);
	BC_EditConditionsFrame_ConditionFrame_SetState(2);
end
--===================================================================================================================--

--State Functions

--===================================================================================================================--
function BC_EditConditionsFrame_SetState(state)
	if(state==0) then
		BC_EditConditionsFrame:Hide();
	elseif(state==1) then
		BC_EditConditionsFrame:Show();
		BC_EditConditionsFrame_ConditionFrame_ListHighlight:SetWidth(180);
	end
end

function BC_EditConditionsFrame_ConditionFrame_SetState(state)
	if(state==0) then
		BC_EditConditionsFrame_ConditionFrame:Hide();
	else
		BC_EditConditionsFrame_ConditionFrame:Show();
		if(state==1) then
			BC_EditConditionsFrame_DeleteButton:Hide();
		elseif(state==2) then
			BC_EditConditionsFrame_DeleteButton:Show();
		end
	end
end

function BC_EditConditionsFrame_ConditionStatus_Set(valid)
	if(valid==true) then
		BC_EditConditionsFrame_ConditionStatus:Show();
		BC_EditConditionsFrame_ConditionStatus:SetTexture("Interface\\Icons\\Spell_ChargePositive");
		BC_Conditions.valid = true;
	elseif(valid==false) then		
		BC_EditConditionsFrame_ConditionStatus:Show();
		BC_EditConditionsFrame_ConditionStatus:SetTexture("Interface\\Icons\\Spell_ChargeNegative");
		BC_Conditions.valid = false;
	else
		BC_EditConditionsFrame_ConditionStatus:Hide();
		BC_Conditions.valid = false;
	end
end

function BC_EditConditionsFrame_HelpText_SetText(index)
	BC_Debug("BC_EditConditionsFrame_HelpText_SetText(" .. index .. ")", 2);
	if(index==1) then
		message = "";
		for i=1, table.getn(BC_CONDITION_TARGETS) do
			message = message .. BC_CONDITION_TARGETS[i] .. "\n";
		end
		BC_EditConditionsFrame_HelpTitle:SetText("Target Values");
		BC_EditConditionsFrame_HelpText:SetText(message);
	elseif(index==2) then
		message = "";
		for i=1, 21 do
			message = message .. BC_CONDITION_VARIABLES[i] .. " = " .. BC_TEXT_HELPVARS[i] .. "\n";
		end
		BC_EditConditionsFrame_HelpTitle:SetText("Variable Values");
		BC_EditConditionsFrame_HelpText:SetText(message);
	elseif(index==3) then
		message = "";
		for i=1, table.getn(BC_CONDITION_TESTS) do
			message = message .. BC_CONDITION_TESTS[i] .. " (" .. BC_CONDITION_TESTS_MEANING[i] .. ")" .. "\n";
		end
		BC_EditConditionsFrame_HelpTitle:SetText("Test Values");
		BC_EditConditionsFrame_HelpText:SetText(message);
	elseif(index==4) then
		BC_EditConditionsFrame_HelpTitle:SetText("Values");
		BC_EditConditionsFrame_HelpText:SetText("");
	end
	BC_EditConditionsFrame_HelpText:SetJustifyV("TOP");
	BC_EditConditionsFrame_HelpText:SetJustifyH("LEFT");
end

--===================================================================================================================--

--Button Functions

--===================================================================================================================--
function BC_EditConditionsFrame_TestButton_OnClick()
	BC_Debug("BC_EditConditionsFrame_TestButton_OnClick()", 2);
	BC_Conditions.valid = BC_EditConditionsFrame_EditBox_ValidateFields();
	BC_EditConditionsFrame_ConditionStatus_Set(BC_Conditions.valid);
	if(BC_Conditions.valid==false) then
		BC_Message(BC_TEXT_CONDITIONTESTFAIL01);
		return;
	end
	condition = BC_EditConditionsFrame_EditBoxValues_Get();
	if(condition.target == "event") then
		BC_Message(BC_TEXT_CONDITIONTESTFAIL02);
		return;
	end
	message = gsub(BC_TEXT_CONDITIONTEST, "XTARGETX", condition.target);
	for i=1, table.getn(BC_CONDITION_VARIABLES) do
		if(condition.variable == BC_CONDITION_VARIABLES[i]) then
			varNum = i;
		end
	end
	message = gsub(message, "XVARIABLEX", BC_TEXT_HELPVARS[varNum]);
	for i=1, table.getn(BC_CONDITION_TESTS) do
		if(condition.test == BC_CONDITION_TESTS[i]) then
			testNum = i;
		end
	end
	message = gsub(message, "XTESTX", BC_CONDITION_TESTS_MEANING[testNum]);
	message = gsub(message, "XVALUEX", condition.value);
	if(BC_TestCondition(condition)) then
		message = gsub(message, "XRESULTX", "true");
	else
		message = gsub(message, "XRESULTX", "false");
	end
	BC_Message(message);
end

function BC_EditConditionsFrame_DeleteButton_OnClick()
	BC_Debug("BC_EditConditionsFrame_DeleteButton_OnClick()", 2);
	if(BC_Selected.condition > 0) then
		tremove(BC_Conditions.ready, BC_Selected.condition);
		BC_Selected.condition = 0;
		BC_EditConditionsFrame_ConditionFrame_SetState(1);
		BC_EditConditionFrame_ConditionFrame_Populate();
	end
end

function BC_EditConditionsFrame_SaveButton_OnClick()
	BC_Debug("BC_EditConditionsFrame_SaveButton_OnClick()", 2);
	BC_Conditions.valid = BC_EditConditionsFrame_EditBox_ValidateFields();
	BC_EditConditionsFrame_ConditionStatus_Set(BC_Conditions.valid);
	if(BC_Conditions.valid) then
		conditionInfo = BC_EditConditionsFrame_EditBoxValues_Get();
		tinsert(BC_Conditions.ready, conditionInfo);		
	end
	BC_Selected.condition = 0;
	BC_EditConditionsFrame_ConditionFrame_SetState(1);
	BC_EditConditionFrame_ConditionFrame_Populate();
end

function BC_EditConditionsFrame_AcceptButton_OnClick()
	BC_Debug("BC_EditConditionsFrame_AcceptButton_OnClick()", 2);
	BC_Conditions.start = {};
	for i=1, table.getn(BC_Conditions.ready) do
		BC_Conditions.start[i] = BC_Conditions.ready[i];
	end
	if(BC_Conditions.ready) then
		CryInputFrame_ConditionButton:SetText(BC_BUTTONLABEL_CONDITIONS .. " (" .. table.getn(BC_Conditions.ready) .. ")");
	else
		CryInputFrame_ConditionButton:SetText(BC_BUTTONLABEL_CONDITIONS);
	end
	BC_Conditions.exclusive = BC_EditConditionsFrame_ExclusiveCheckbox:GetChecked();
	BC_Selected.condition = 0;
	BC_EditConditionsFrame_SetState(0);
	BC_CriesFrame_SetState(1);
end

function BC_EditConditionsFrame_ResetButton_OnClick()
	BC_Debug("BC_EditConditionsFrame_ResetButton_OnClick()", 2);
	BC_Conditions.ready = {};
	for i=1, table.getn(BC_Conditions.start) do
		BC_Conditions.ready[i] = BC_Conditions.start[i];
	end
	BC_EditConditionsFrame_ExclusiveCheckbox:SetChecked(BC_Conditions.exclusive);
	BC_Selected.condition = 0;
	BC_EditConditionFrame_ConditionFrame_Populate();
end

function BC_EditConditionsFrame_CancelButton_OnClick()
	BC_Debug("BC_EditConditionsFrame_CancelButton_OnClick()", 2);
	BC_Conditions.ready = {};
	for i=1, table.getn(BC_Conditions.start) do
		BC_Conditions.ready[i] = BC_Conditions.start[i];
	end
	BC_EditConditionsFrame_ExclusiveCheckbox:SetChecked(BC_Conditions.exclusive);
	BC_Selected.condition = 0;
	BC_EditConditionsFrame_SetState(0);
	BC_CriesFrame_SetState(1);
end

--===================================================================================================================--

--EditBox Functions

--===================================================================================================================--

function BC_EditConditionsFrame_EditBoxValues_Set(conditionInfo)
	BC_Debug("BC_EditConditionsFrame_EditBoxValues_Set(...)", 2);
	getglobal("BC_EditConditionsFrame_VarEditBox"):SetText(conditionInfo.variable);
	getglobal("BC_EditConditionsFrame_TesEditBox"):SetText(conditionInfo.test);
	getglobal("BC_EditConditionsFrame_ValEditBox"):SetText(conditionInfo.value);
	getglobal("BC_EditConditionsFrame_TarEditBox"):SetText(conditionInfo.target);
end

function BC_EditConditionsFrame_EditBoxValues_Get()
	BC_Debug("BC_EditConditionsFrame_EditBoxValues_Get()", 2);
	local conditionInfo = {};
	conditionInfo.variable =getglobal("BC_EditConditionsFrame_VarEditBox"):GetText();
	conditionInfo.test = 	getglobal("BC_EditConditionsFrame_TesEditBox"):GetText();
	conditionInfo.value = 	getglobal("BC_EditConditionsFrame_ValEditBox"):GetText();
	conditionInfo.target = 	getglobal("BC_EditConditionsFrame_TarEditBox"):GetText();
	return conditionInfo;
end

function BC_EditConditionsFrame_EditBox_FocusNext(this)
	BC_Debug("BC_EditConditionsFrame_EditBox_FocusNext(" .. this:GetName() .. ")", 2);
	thisBox = strsub(this:GetName(),strlen(this:GetName())-9);
	if(thisBox == "TarEditBox") then
		nextBox = "VarEditBox"
	elseif(thisBox == "VarEditBox") then
		nextBox = "TesEditBox"
	elseif(thisBox == "TesEditBox") then
		nextBox = "ValEditBox"
	elseif(thisBox == "ValEditBox") then
		nextBox = "TarEditBox"
	end
	getglobal(this:GetParent():GetName() .. "_" .. nextBox):SetFocus();
end

function BC_EditConditionsFrame_EditBox_ValidateText(text, checkTable)
	BC_Debug("BC_EditConditionsFrame_EditBox_ValidateText(" .. text .. ", ...)", 2);
	if(strsub(text,1,2)=="$e") then text = "$e#"; end
	flag = 0;
	for i=1, table.getn(checkTable) do
		if(text == checkTable[i]) then
			flag = i;
		end
	end
	return flag;
end

function BC_EditConditionsFrame_EditBox_ValidateValue(text, varType, testType)
	BC_Debug("BC_EditConditionsFrame_EditBox_ValidateValue(" .. text .. ", " .. varType .. ", " .. tostring(testType) .. ")", 2);
	if(text=="" or text==nil) then return 0; end
	if(BC_CONDITION_TYPES[varType]==2) then
		BC_Debug(" Found a number (" .. tostring(tonumber(text)) .. ") at " .. tostring(i));
		if(tostring(tonumber(text))=="nil") then
			return 0;
		end
	elseif(BC_CONDITION_TYPES[varType]==3) then
		BC_Debug(" Found a boolean at " .. tostring(i));
		if(strlower(text)~="true" and strlower(text) ~="false") then
			return 0;
		end
		if(testType ~= "==" and testType ~= "~=") then
			return 0;
		end
	end
	return 1;
end

function BC_EditConditionsFrame_EditBox_ValidateFields()
	BC_Debug("BC_EditConditionsFrame_EditBox_ValidateFields()", 2);
	conditionInfo = {};
	tarBox = strlower(getglobal("BC_EditConditionsFrame_TarEditBox"):GetText());
	varBox = strlower(getglobal("BC_EditConditionsFrame_VarEditBox"):GetText());
	tesBox = strlower(getglobal("BC_EditConditionsFrame_TesEditBox"):GetText());
	valBox = strlower(getglobal("BC_EditConditionsFrame_ValEditBox"):GetText());
	if( tarBox == "" and
		varBox == "" and
		tesBox == "" and
		valBox == "") then
		return false;
	end
	tarStat = BC_EditConditionsFrame_EditBox_ValidateText(tarBox, BC_CONDITION_TARGETS);
	varStat = BC_EditConditionsFrame_EditBox_ValidateText(varBox, BC_CONDITION_VARIABLES);
	tesStat = BC_EditConditionsFrame_EditBox_ValidateText(tesBox, BC_CONDITION_TESTS);
	valStat = BC_EditConditionsFrame_EditBox_ValidateValue(valBox, varBox, BC_CONDITION_TESTS[tesStat]);
	if((tarStat * varStat * tesStat * valStat)>0) then
		return true;
	else
		return false;
	end
end

function BC_EditConditionsFrame_EditBox_OnEditFocusLost(this)
	BC_Debug("BC_EditConditionsFrame_EditBox_OnEditFocusLost(" .. this:GetName() .. ")", 2);
	BC_Conditions.valid = BC_EditConditionsFrame_EditBox_ValidateFields();
	BC_EditConditionsFrame_ConditionStatus_Set(BC_Conditions.valid);
end

--===================================================================================================================--

--Unit Functions

--===================================================================================================================--

function BC_UnitHealthPercent(target)
	if(target) then
		healthMax = UnitHealthMax(target);
		healthCur = UnitHealth(target);
		return tonumber((healthCur / healthMax)*100);		
	else
		return nil;
	end
end

function BC_UnitManaPercent(target)
	if(target) then
		manaMax = UnitManaMax(target);
		manaCur = UnitMana(target);
		if(manaCur == 0) then
			return 0;
		end
		return tonumber((manaCur / manaMax)*100);		
	else
		return nil;
	end
end

function BC_GuildInfo(target)
	name, _, _ = GetGuildInfo(target);
	return name;
end

function BC_UnitReaction(target)
	return UnitReaction(target, "player");
end

function BC_UnitInParty(target)
	if(target == "player") then
		if(GetNumPartyMembers() > 0) then
			return true;
		else
			return false;
		end
	elseif(UnitInParty(target) == 1) then
		return true;
	else
		return false;
	end
end

function BC_UnitInRaid(target)
	if(target == "player") then
		if(GetNumRaidMembers() > 0) then
			return true;
		else
			return false;
		end
	elseif(UnitInRaid(target) == 1) then
		return true;
	else
		return false;
	end
end

function BC_UnitItemMH()
	itemLink = GetInventoryItemLink("player",16);
	if(itemLink) then
		itemName,_,_,_,_,_,_,_,_,_ = GetItemInfo(itemLink);
		return itemName;
	end
	return nil;
end

function BC_UnitItemOH()
	itemLink = GetInventoryItemLink("player",17);
	if(itemLink) then
		itemName,_,_,_,_,_,_,_,_,_ = GetItemInfo(itemLink);
		return itemName;
	end
	return nil;
end

function BC_UnitItemRW()
	itemLink = GetInventoryItemLink("player",18);
	if(itemLink) then
		itemName,_,_,_,_,_,_,_,_,_ = GetItemInfo(itemLink);
		return itemName;
	end
	return nil;
end

--===================================================================================================================--

-- Condition Functions

--===================================================================================================================--
function BC_TestCondition(condition)
	BC_Debug("BC_TestCondition(..)", 2);
	if(condition.target == "event") then
		condition.target = tonumber(strsub(condition.variable,3,3));
		condition.variable = "$e#";
	end
	for i=1, table.getn(BC_CONDITION_VARIABLES) do
		if(condition.variable == BC_CONDITION_VARIABLES[i]) then
			BC_Debug(" Found at " .. i .. " with " .. BC_CONDITION_FUNCTIONS[i]);
			if(BC_CONDITION_VARIABLES[i] == "$nm" and condition.target == "target" and BC_Loaded.target) then
				retval = BC_Loaded.target;
			else
				func = getglobal(BC_CONDITION_FUNCTIONS[i]);
				_, retval = pcall(func,condition.target);
			end
			BC_Debug(" " .. tostring(retval) .. condition.test .. condition.value);
			if(BC_CONDITION_TYPES[i]==2) then
				condition.value = tonumber(condition.value);
			end
			if(BC_CONDITION_TYPES[i]==3 and retval == nil) then
				retval = "false";
			end
			if(condition.test == BC_CONDITION_TESTS[1]) then
				if(retval == condition.value) then return true;
				else return false; end
			elseif(condition.test == BC_CONDITION_TESTS[2]) then
				if(retval < condition.value) then return true;
				else return false; end
			elseif(condition.test == BC_CONDITION_TESTS[3]) then
				if(retval <= condition.value) then return true;
				else return false; end
			elseif(condition.test == BC_CONDITION_TESTS[4]) then
				if(retval > condition.value) then return true;
				else return false; end;
			elseif(condition.test == BC_CONDITION_TESTS[5]) then
				if(retval >= condition.value) then return true;
				else return false; end
			elseif(condition.test == BC_CONDITION_TESTS[6]) then
				if(retval ~= condition.value) then return true;
				else return false; end
			elseif(condition.test == BC_CONDITION_TESTS[7]) then
				if(strfind(retval, condition.value)) then return true;
				else return false; end
			elseif(condition.test == BC_CONDITION_TESTS[8]) then
				if(strfind(strlower(retval), strlower(condition.value))) then return true;
				else return false; end
			else
				return false;
			end
		end
	end
	return false;
end