--===================================================================================================================--

-- BC_Saved Functions

--===================================================================================================================--
function BC_SavedActions_Add(action)
	BC_Debug("BC_SavedActions_Add(" .. action .. ")", 2);
	stillAdd = true;
	if(table.getn(BC_Saved.actions) > 0) then
		for i=1, table.getn(BC_Saved.actions) do
			if(BC_Saved.actions[i] == action) then
				stillAdd = false;
			end
		end
	end
	if(stillAdd) then
		tinsert(BC_Spellbook[BC_VARS_CATEGORYCUSTOM], action);
		tinsert(BC_Saved.actions, action);
		BC_Saved.cries[action] = { frequency = 0, cooldown = 0, cry = {},};
	end
end

function BC_SavedActions_Delete(action)
	BC_Debug("BC_SavedActions_Delete(" .. action .. ")", 2);
	if(action ~= nil) then
		tremove(BC_Spellbook[BC_VARS_CATEGORYCUSTOM], action);
		tremove(BC_Saved.actions, action);
		tremove(BC_Saved.cries, action);
	end
end

function BC_SavedCries_Delete(actionName, cryNum)
	BC_Debug("BC_SavedCries_Delete(" .. actionName .. ", " .. cryNum .. ")", 2);
	if(actionName ~= nil and cryNum ~= nil) then
		message = gsub(BC_TEXT_REMOVECRY, "XACTIONX", actionName);
		message = gsub(message, "XCRYX", BC_Saved.cries[actionName].cry[cryNum].name);
		BC_Message(message);
		tremove(BC_Saved.cries[actionName].cry, cryNum);
		if(BC_Saved.events[actionName]) then
			BC_EventCry_Unregister(BC_Saved.events[actionName].event);
		end
	end
end

function BC_SavedCries_Save(actionName, cryNum, cry)
	BC_Debug("BC_SavedCries_Save(" .. actionName .. ", " .. cryNum .. ", ...)", 2);
	if(actionName ~= nil and cry ~= nil) then
		message = gsub(BC_TEXT_SAVECRY, "XACTIONX", actionName);
		message = gsub(message, "XCRYX", cry.name);
		BC_Message(message);
		if(cryNum==0) then
			if(BC_Saved.cries[actionName].cry) then
				for i=1, table.getn(BC_Saved.cries[actionName].cry) do
					if(BC_Saved.cries[actionName].cry[i].name==cry.name) then
						cryNum = i;
					end
				end
			end
		end
		if(cryNum > 0 and (cry.name == BC_CryListDB[cryNum])) then
			BC_Saved.cries[actionName].cry[cryNum] = {
				name = cry.name,
				text = cry.text,
				mode = cry.mode,
				language = cry.language,
				castCode = cry.castCode,
				exclude = cry.exclude,
				exclusive = cry.exclusive,
				always = cry.always,
				frequency = cry.frequency,
				condition = cry.condition,			};
		else
			if(BC_CategoryDB[BC_Selected.category] == BC_VARS_CATEGORYEVENTS) then
				BC_EventCry_Register(BC_Saved.events[actionName].event);
			end
			tinsert(BC_Saved.cries[actionName].cry, {
				name = cry.name,
				text = cry.text,
				mode = cry.mode,
				language = cry.language,
				castCode = cry.castCode,
				exclude = cry.exclude,
				exclusive = cry.exclusive,
				always = cry.always,
				frequency = cry.frequency,
				condition = cry.condition,			});
		end				
	else
		BC_Message(BC_TEXT_SAVECRY_FAIL01);
	end
end

function BC_SavedCries_SetFrequency(frequency, action)
	BC_Debug("BC_SetFrequency(" .. tostring(frequency) .. ", " .. tostring(action) .. ")",2);
	if(not action) then
		action = BC_ActionDB[BC_Selected.action];
	end
	BC_Saved.cries[action].frequency = frequency;
end

function BC_SavedCries_SetCooldown(cooldown, action)
	BC_Debug("BC_SetCooldown(" .. tostring(cooldown) .. ", " .. tostring(action) .. ")",2);
	if(not action) then
		action = BC_ActionDB[BC_Selected.action];
	end
	if(cooldown == nil or cooldown == "") then cooldown = 0; end
	BC_Saved.cries[action].cooldown = tonumber(cooldown);
end

function BC_SavedCries_GetCooldown(action)
	BC_Debug("BC_GetCooldown(" .. tostring(action) .. ")",2);
	if(not action) then
		action = BC_ActionDB[BC_Selected.action];
	end
	return BC_Saved.cries[action].cooldown;
end


--===================================================================================================================--

-- Action Functions

--===================================================================================================================--
function BC_SetCooldown(actionName)
	BC_Debug("BC_SetCooldown(" .. actionName .. ")", 2);
	if(tonumber(BC_Saved.cries[actionName].cooldown) > 0) then
		BC_Debug(actionName .. " is now on cooldown.", 3);
		tinsert(BC_Loaded.cooldowns,
			{["actionName"] = actionName, 
			 ["time"] = GetTime() + BC_Saved.cries[actionName].cooldown,});
	end
end

function BC_CheckCooldown(actionName)
	BC_Debug("BC_CheckCooldown(" .. tostring(actionName) .. ")", 2);
	if(not actionName) then
		return 0;
	end
	if(not BC_Saved.cries[actionName]) then
		return 0;
	end
	if(tonumber(BC_Saved.cries[actionName].cooldown) > 0) then
		now = GetTime();
		BC_Debug( actionName .. " has a cooldown of " .. tostring(BC_Saved.cries[actionName].cooldown) .. " seconds.", 3);
		for i=1, table.getn(BC_Loaded.cooldowns) do
			if(BC_Loaded.cooldowns[i].actionName == actionName) then
				if(BC_Loaded.cooldowns[i].time < now) then
					BC_Debug(actionName .. " was on cooldown but is no longer.", 3);
					tremove(BC_Loaded.cooldowns,i);
					return 2;
				else
					BC_Debug(actionName .. " is still on cooldown.", 3);
					return 0;
				end
			end
		end
		return 2;
	end
	return 1;
end

function BC_ActionManagement(actionName, castCode)
	BC_Debug("BC_ActionManagement(" .. tostring(actionName) .. ", " .. castCode .. ")",2);
	if(not BC_Vars.disabled) then
		if(actionName) then
			if(BC_Saved.cries[actionName]) then
				if(BC_Vars.echo) then
					BC_Message(BC_TEXT_ECHO .. actionName);
				end
				if(BC_CheckCooldown(actionName) > 0) then
					BC_Loaded.cry = BC_SelectCry(BC_ClearQualifiers(actionName), castCode);
				end
			else
				BC_Loaded.cry = {};
				BC_Loaded.chain = {};
			end
		end
	end
end

function BC_ActionStart()
	BC_Debug("BC_ActionStart()",2);
	BC_ActionManagement(BC_Loaded.action, 2);
	cooldown = BC_CheckCooldown(BC_Loaded.action);
	if(cooldown > 0) then
		if(BC_Loaded.cry) then
			if(cooldown == 2) then
				BC_SetCooldown(BC_Loaded.action);
			end
			BC_ExecuteCry(BC_Loaded.cry);
		end
	end
end

function BC_SetAction(actionName)
	BC_Debug("BC_SetAction(" .. actionName .. ")",2);
	BC_Loaded.action = actionName;
end

function BC_ActionStop()
	BC_Debug("BC_ActionStop()",2);
end

function BC_ActionSucceeded(actionName)
	BC_Debug("BC_ActionSucceeded(" .. tostring(actionName) .. ")",2);
	BC_ActionManagement(actionName, 1);
	if(BC_Loaded.cry) then
		cooldown = BC_CheckCooldown(actionName);
		if(cooldown > 0) then
			if(cooldown == 2) then
				BC_SetCooldown(actionName);
			end
			BC_ExecuteCry(BC_Loaded.cry);
		end
	end
end

function BC_ActionInterrupted()
	BC_Debug("BC_ActionInterrupted()",2);
	BC_ActionManagement(BC_Loaded.action, 3);
	if(BC_Loaded.cry) then
		cooldown = BC_CheckCooldown(actionName);
		if(cooldown > 0) then
			if(cooldown == 2) then
				BC_SetCooldown(actionName);
			end
			BC_ExecuteCry(BC_Loaded.cry);
		end
	end
end

--===================================================================================================================--

-- Cry Functions

--===================================================================================================================--
function BC_CompileChains(actionName, cryNum)
	BC_Debug("BC_CompileChains(" .. actionName .. ", " .. cryNum .. ")", 2);
	BC_Loaded.chain = {};
	for i=1, table.getn(BC_Saved.chains) do
		if(BC_Saved.chains[i].actionName1 == actionName and BC_Saved.chains[i].cry1 == cryNum) then
			tinsert(BC_Loaded.chain, {nextAction = BC_Saved.chains[i].actionName2, nextCry = BC_Saved.chains[i].cry2});
		end
	end
end

function BC_TestCryConditions(actionName, cryNum)
	BC_Debug("BC_TestCryConditions(" .. actionName .. ", " .. tostring(cryNum) .. ")", 2);
	numConditions = table.getn(BC_Saved.cries[actionName].cry[cryNum].condition);
	for j=1, numConditions do
		if(BC_TestCondition(BC_Saved.cries[actionName].cry[cryNum].condition[j]) == false) then
			BC_Debug(" condition failed at " .. cryNum .. ", " .. j);
			return false;
		end
	end
	return true;
end

function BC_TestExclusive(actionName, cryNum)
	BC_Debug("BC_TestExclusive(" .. actionName .. ", " .. tostring(cryNum) .. ")", 2);
	if(BC_Saved.cries[actionName].cry[cryNum].exclusive == 1) then
		return true;
	else
		return false;
	end
end

function BC_SelectCry(actionName, castCode)
	BC_Debug("BC_SelectCry(" .. tostring(actionName) .. ", " .. tostring(castCode) .. ")",2);
	if(BC_Saved.cries[actionName]) then
		-- Compile eligible cries
		numCries = table.getn(BC_Saved.cries[actionName].cry);
		availableCries = { };
		conditionCries = { };
		exclusiveCries = { };
		standardCries = { };
		alwaysCries = { };
		exclusive = false;
		chained = false;
		shouldCry = random(1,100);
		if(BC_Loaded.chain) then
			for i=1, table.getn(BC_Loaded.chain) do
				if(BC_Loaded.chain[i].nextAction == actionName) then
					BC_Debug("Cry chained.", 3);
					tinsert(alwaysCries, BC_Loaded.chain[i].nextCry);
					chained = true;
				end
			end
		end
		if(chained == false) then
			BC_Debug(" numCries = " .. tostring(numCries),3);
			for i=1, numCries do
				if(BC_Saved.cries[actionName].cry[i].castCode == castCode) then
					if(BC_Saved.cries[actionName].cry[i].exclude == 0) then
						if(BC_Saved.cries[actionName].cry[i].condition) then
							numCondition = table.getn(BC_Saved.cries[actionName].cry[i].condition);
						else
							numCondition = 0;
						end
						if(numCondition > 0) then
							passedCondition = BC_TestCryConditions(actionName, i);
							if(passedCondition == true) then
								BC_Debug("  Condition OK for " .. i, 3);
								if(BC_Saved.cries[actionName].cry[i].exclusive == 1) then
									exclusive = true;
									if(BC_Saved.cries[actionName].cry[i].always == 1) then
										for j=1, BC_Saved.cries[actionName].cry[i].frequency do
											tinsert(exclusiveCries, i);
										end
									end
									for j=1, BC_Saved.cries[actionName].cry[i].frequency do
										tinsert(conditionCries, i);
									end
								else
									if(BC_Saved.cries[actionName].cry[i].always == 1) then
										for j=1, BC_Saved.cries[actionName].cry[i].frequency do
											tinsert(alwaysCries, i);
										end
									end
									for j=1, BC_Saved.cries[actionName].cry[i].frequency do
										tinsert(standardCries, i);
									end
								end
							end
						else
							BC_Debug("  No condition for " .. i, 3);
							if(BC_Saved.cries[actionName].cry[i].always == 1) then
								for j=1, BC_Saved.cries[actionName].cry[i].frequency do
									tinsert(alwaysCries, i);
								end
							end
							for j=1, BC_Saved.cries[actionName].cry[i].frequency do
								tinsert(standardCries, i);
							end
						end
					end
				end
			end
		end
		BC_Debug("  Frequency set to " .. BC_Saved.cries[actionName].frequency .. " and rolled " .. shouldCry, 3);
		if(shouldCry <= BC_Saved.cries[actionName].frequency) then
			if(exclusive) then
				if(table.getn(conditionCries)>0) then
					for i=1, table.getn(conditionCries) do
						tinsert(availableCries, conditionCries[i]);
					end
				end
			else
				if(table.getn(standardCries)>0) then
					for i=1, table.getn(standardCries) do
						tinsert(availableCries, standardCries[i]);
					end
				end
			end
		else
			if(exclusive) then
				if(table.getn(exclusiveCries)>0) then
					for i=1, table.getn(exclusiveCries) do
						tinsert(availableCries, exclusiveCries[i]);
					end
				end
			else
				if(table.getn(alwaysCries)>0) then
					for i=1, table.getn(alwaysCries) do
						tinsert(availableCries, alwaysCries[i]);
					end
				end
			end
		end
		if(table.getn(availableCries)>0) then
			cryNum = random(1,table.getn(availableCries));
			thisCry = BC_GetCry(actionName, availableCries[cryNum]);
			if(BC_CheckCry(thisCry.text)==1) then
				BC_CompileChains(actionName, availableCries[cryNum]);
				return thisCry;
			end
		end
	else
		BC_Debug("No cries for " .. actionName .. " saved in BC_Saved.", 3);
	end
	BC_Debug("No acceptable cries found.", 3);
	BC_Loaded.chain = {};
	return nil;
end

function BC_GetCry(actionName, cryNum)
	BC_Debug("BC_GetCry(" .. tostring(actionName) .. ", " .. tostring(cryNum) .. ")",2);
	local thisCry = {
		["mode"] = 		BC_Saved.cries[actionName].cry[cryNum].mode,
		["text"] = 		BC_Saved.cries[actionName].cry[cryNum].text, 
		["language"] = 	BC_Saved.cries[actionName].cry[cryNum].language,
		["start"] =		BC_Saved.cries[actionName].cry[cryNum].start, 
		["actionName"] = actionName,};
	return BC_FormatCry(thisCry);
end

function BC_FormatCry(thisCry)
	BC_Debug("BC_FormatCry(..)",2);
	thisCry.language = GetLanguageByIndex(thisCry.language);
	if(thisCry.mode == 1 ) then
		thisCry.mode = "SAY";
	elseif(thisCry.mode == 2) then
		thisCry.mode = "EMOTE";
	elseif(thisCry.mode == 3) then
		thisCry.mode = "YELL";
	elseif(thisCry.mode == 4) then
		if(GetNumPartyMembers() > 0) then
			thisCry.mode = "PARTY";
		else
			thisCry.mode = "SAY";
		end
	else
		if(GetNumRaidMembers() > 0) then
			if(thisCry.mode == 6) then
				if(IsRaidLeader() or IsRaidOfficer()) then
					thisCry.mode = "RAIDWARNING";
				else
					thisCry.mode = "RAID";
				end
			else
				thisCry.mode = "RAID";
			end			
		elseif(GetNumPartyMembers() > 0) then
			thisCry.mode = "PARTY";
		else
			thisCry.mode = "SAY";
		end
	end
	for i=1,table.getn(BC_CONDITION_VARIABLES) do
		if(strfind(thisCry.text, BC_CONDITION_VARIABLES[i])) then
			BC_Debug(tostring(BC_Loaded.target),3);
			if(BC_CONDITION_VARIABLES[i] == "$nm" and BC_Loaded.target) then
				thisCry.text = gsub(thisCry.text, BC_CONDITION_VARIABLES[i], BC_Loaded.target);
			else
				func = getglobal(BC_CONDITION_FUNCTIONS[i]);
				if(BC_CONDITION_VARIABLES[i]=="$pn") then
					target = "pet";
				else
					target = "target";
				end
				_, retval = pcall(func,target);
				BC_Debug("Calling " .. BC_CONDITION_FUNCTIONS[i] .. "(" .. target .. ") returned " .. tostring(retval), 3);
				if(retval ~= nil) then
					if(BC_CONDITION_VARIABLES[i]=="$po") then
						replaceText = BC_VARS_PROO[retval];
					elseif(BC_CONDITION_VARIABLES[i]=="$pp") then
						replaceText = BC_VARS_PROP[retval];
					elseif(BC_CONDITION_VARIABLES[i]=="$ps") then
						replaceText = BC_VARS_PROS[retval];
					elseif(BC_CONDITION_VARIABLES[i]=="$ga") then
						replaceText = BC_VARS_GEN1[retval];
					elseif(BC_CONDITION_VARIABLES[i]=="$gb") then
						replaceText = BC_VARS_GEN2[retval];
					else
						replaceText = retval;
					end
					thisCry.text = gsub(thisCry.text, BC_CONDITION_VARIABLES[i], replaceText);
				end
			end
		end
	end
	thisCry.text = BC_EventVariables_Replace(thisCry.text);
	endNewLine = true;

	while(endNewLine) do
		if(strsub(thisCry.text,strlen(thisCry.text))=="\n") then
			BC_Debug("Pulling new line from end of string: '" .. thisCry.text .. "'", 3);
			thisCry.text = strsub(thisCry.text,1,strlen(thisCry.text)-1);
		else
			BC_Debug("No more new lines at end of string: '" .. thisCry.text .. "'", 3);
			endNewLine = false;
		end
	end
	return thisCry;
end

function BC_CheckCry(cryText)
	BC_Debug("BC_CheckCry(" .. cryText .. ")", 2);
	if(not cryText) then
		BC_Debug(" CheckCry failed: There is no cry.", 3);
		return 0;
	end
	for i=1,table.getn(BC_CONDITION_VARIABLES) do
		if(strfind(cryText,BC_CONDITION_VARIABLES[i])) then
			func = getglobal(BC_CONDITION_FUNCTIONS[i]);
			if(BC_CONDITION_VARIABLES[i]=="$pn") then
				target = "pet";
			else
				target = "target";
			end
			_, retval = pcall(func,target);
			if(not retval) then
				BC_Debug(" CheckCry failed: " .. BC_CONDITION_VARIABLES[i] .. " called " .. BC_CONDITION_FUNCTIONS[i], 3);
				return 0;
			else
				BC_Debug(" CheckCry Passed", 3);
				return 1;
			end
		end
	end
	BC_Debug(" CheckCry Passed", 3);
	return 1;
end

function BC_ExecuteCry(thisCry)
	BC_Debug("BC_ExecuteCry(...)",2);
	if(thisCry) then
		if(not thisCry.text) then
			BC_Debug("No cry text.");
			return nil;
		end
		if(strfind(thisCry.text,"\n")) then
			multiLine = true;
			while(multiLine) do
				endLine = strfind(thisCry.text,"\n");
				thisLine = strsub(thisCry.text,1, endLine-1);
				BC_SendChatMessage(thisLine, thisCry.mode, thisCry.language);
				thisCry.text = strsub(thisCry.text,endLine+1,strlen(thisCry.text));
				BC_Debug("thisCry.text: |" .. thisCry.text .. "|");
				if(not strfind(thisCry.text,"\n")) then multiLine = false; end
			end
			BC_SendChatMessage(thisCry.text, thisCry.mode, thisCry.language);
		else
			BC_SendChatMessage(thisCry.text, thisCry.mode, thisCry.language);
		end
	else
		BC_Debug("No cry inputed.", 3);
	end
	BC_Loaded.action = nil;
	BC_Loaded.cry = {};
	BC_Loaded.target = nil;
end

function BC_SendChatMessage(text, mode, language)
	BC_Debug("BC_SendChatMessage(...)", 2);
	if(BC_Vars.debug) then
		BC_Debug("|CFFB0B0B0[" .. tostring(mode) .. "] (" .. tostring(language) .. ") " .. tostring(text) .. "|r");
	else
		if(strsub(text,1,1)=="/") then
			BC_Debug("Slash command found: " .. text);
			BC_MainFrame_ExecBox:SetText(text);
			ChatEdit_SendText(BC_MainFrame_ExecBox);
			BC_MainFrame_ExecBox:SetText("");
		else		
			SendChatMessage(text, mode, language);
		end
	end
end

function BC_DisplayEmotes()
	i = 1;
	BC_Message("Available emotes:");
	message = "";
	listToken = getglobal("EMOTE" .. tostring(i) .. "_TOKEN");
	while(listToken) do
		message = message .. strlower(listToken);
		i = i + 1;
		listToken = getglobal("EMOTE" .. tostring(i) .. "_TOKEN");
		if(mod(i,10) == 0) then
			BC_Message(message);
			message = "";
		else
			message = message .. ", ";
		end
	end
end
