--===================================================================================================================--

-- On Login Functions

--===================================================================================================================--

function BC_MainFrame_OnLoad()
	SLASH_BATTLECRY1 = "/bc";
	SLASH_BATTLECRY2 = "/battlecry";
	SlashCmdList["BATTLECRY"] = BC_SlashHandler;

	hooksecurefunc("CastPetAction", BC_CastPetAction)
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("LEARNED_SPELL_IN_TAB");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_ALIVE");
end

function BC_MainFrame_OnEvent()
	BC_Debug(event .. ", " .. tostring(arg1) .. ", " .. tostring(arg2) .. ", " .. tostring(arg3),1);
	if(event=="ADDON_LOADED") then
		BC_Debug(arg1,1);
		if(arg1=="BattleCry") then
			BC_Message(BC_TEXT_ADDONLOADED);
			this:UnregisterEvent(event);
		end
	elseif(event=="LEARNED_SPELL_IN_TAB") then
		BC_PopulateSpellbook();
		BC_PopulatePetbook();
		BC_CryEditCategoryFrame_Populate();
	elseif(event=="VARIABLES_LOADED") then
		BC_Debug("Variables Loaded.",1);
		this:UnregisterEvent(event);
	elseif(event=="PLAYER_LOGIN") then
		this:UnregisterEvent(event);
	elseif(event=="PLAYER_ENTERING_WORLD") then
		BC_List_Convert();
		BC_Initialize_DB();
		BC_Initialize_UI();
		this:UnregisterEvent("PLAYER_ENTERING_WORLD");
		this:RegisterEvent("UNIT_SPELLCAST_SENT");
		this:RegisterEvent("UNIT_SPELLCAST_START");
		this:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
		this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
		this:RegisterEvent("LEARNED_SPELL_IN_TAB");
		this:RegisterEvent("PLAYER_PET_CHANGED");
		this:RegisterEvent("PLAYER_ENTER_COMBAT");
	elseif(event=="PLAYER_ALIVE") then
		this:UnregisterEvent(event);
	elseif(event=="PLAYER_ENTER_COMBAT") then
		BC_ActionManagement("Attack", 0);
		BC_ExecuteCry(BC_Loaded.cry);
	elseif(event=="LEARNED_SPELL_IN_TAB") then
		BC_PopulateSpellbook();
		BC_PopulatePetbook();
	elseif(event=="PLAYER_PET_CHANGED") then
		BC_PopulatePetbook();
		BC_CryEditCategoryFrame_Populate();
	elseif(strsub(event,1,5)=="UNIT_") then
		if(arg1=="player" and arg2 ~= "LOGINEFFECT") then
			if(event=="UNIT_SPELLCAST_SUCCEEDED") then
				BC_ActionSucceeded(arg2);
			elseif(event=="UNIT_SPELLCAST_START") then
				BC_ActionStart();
			elseif(event=="UNIT_SPELLCAST_INTERRUPTED") then
				BC_ActionInterrupted();
			elseif(event=="UNIT_SPELLCAST_STOP") then
				BC_ActionStop();
			elseif(event == "UNIT_SPELLCAST_SENT") then
				BC_SetAction(arg2);
				BC_Loaded.target = arg4;
			else
				BC_EventManagement(event);
			end
		end
	else
		BC_EventManagement(event);
	end
end

function BC_Initialize_DB()
	BC_Debug("BC_Initialize_DB()",2);
	BC_Initialize_Events();
	BC_PopulateSpellbook();
	BC_PopulatePetbook();
	BC_PopulateLanguages();
	BC_Authenticate_DB();
end

function BC_Initialize_UI()
	BC_Debug("BC_Initialize_UI()",2);
	CryEditCategoryFrame_TitleText:SetText(BC_FRAMELABEL_CATEGORIES);
	CryEditActionFrame_TitleText:SetText(BC_FRAMELABEL_ACTIONS);
	CryEditCryListFrame_TitleText:SetText(BC_FRAMELABEL_CRIES);
	CryInputFrame_TitleText:SetText(BC_FRAMELABEL_EDITCRY);
	CryEditCustomCryEditActionFrame_TitleText:SetText(BC_FRAMELABEL_CUSTOMS);
	BC_CryEditCategoryFrame_Populate();
	BC_EventWatchFrame_Populate();
	BC_MainFrame_HelpFrame_SetText(getglobal("BC_TabFrameTab3"));
end


--===================================================================================================================--

-- Update Functions

--===================================================================================================================--

function BC_List_Convert()
	BC_Debug("BC_List_Convert()",2);
	if(BC_List) then
		for key in pairs(BC_List) do
			BC_Debug("Converting " .. key, 3);
			BC_Saved.cries[key] = BC_List[key];
			if(BC_Saved.cries[key].cry) then
				for i=1, table.getn(BC_Saved.cries[key].cry) do
					BC_Message("Converting " .. key .. " cry " .. i, 3);
					BC_Saved.cries[key].cry[i].interrupt = 0;
					if(BC_Saved.cries[key].cry[i].exclusive == true) then BC_Saved.cries[key].cry[i].exclusive = 1;
					else BC_Saved.cries[key].cry[i].exclusive = 0; end
					if(BC_Saved.cries[key].cry[i].always == true) then BC_Saved.cries[key].cry[i].always = 1;
					else BC_Saved.cries[key].cry[i].always = 0; end
					BC_Saved.cries[key].cry[i].start = BC_Saved.cries[key].cry[i].post;
					BC_Saved.cries[key].cry[i].post = nil;
					BC_Saved.cries[key].cry[i].condition = BC_Saved.cries[key].cry[i].conditions;
					BC_Saved.cries[key].cry[i].conditions = nil;
					if(not BC_Saved.cries[key].cry[i].exclude) then BC_Saved.cries[key].cry[i].exclude = 0; end
				end
			else
				BC_Saved.cries[key].cry = {};
			end
		end
	end
	if(BC_Vars) then
		if(BC_Vars.Spells) then
			for key in pairs(BC_Vars.Spells) do
				tinsert(BC_Saved.actions,BC_Vars.Spells[key]);
			end
			BC_Vars.Spells = nil;
		end
		if(BC_Vars.chain) then
			for key in pairs(BC_Vars.chain) do
				BC_Vars.chain[key].actionName1 = BC_Vars.chain[key].action1;
				BC_Vars.chain[key].actionName2 = BC_Vars.chain[key].action2;
				BC_Vars.chain[key].action1 = nil;
				BC_Vars.chain[key].action2 = nil;
				tinsert(BC_Saved.chains, BC_Vars.chain[key]);
			end
			BC_Vars.chain = nil;
		end
		BC_Vars.version = BC_VERSION;
		BC_Vars.enabled = nil;
	end
end

function BC_Authenticate_DB()
	BC_Debug("BC_Authenticate_DB()",2);
	for key in pairs(BC_Saved.cries) do
		if(type(BC_Saved.cries[key].cooldown) ~= "number") then
			BC_Saved.cries[key].cooldown = 0;
		end
		if(BC_Saved.cries[key].cry == nil) then
			BC_Saved.cries[key].cry = { };
		end
		if(table.getn(BC_Saved.cries[key].cry) > 0) then
			for key2 in pairs(BC_Saved.cries[key].cry) do
				if(BC_Saved.cries[key].cry[key2].exclusive == nil) then
					BC_Saved.cries[key].cry[key2].exclusive = 0;
				end
				if(BC_Saved.cries[key].cry[key2].always == nil) then
					BC_Saved.cries[key].cry[key2].always = 0;
				end
				if(BC_Saved.cries[key].cry[key2].text == nil) then
					BC_Saved.cries[key].cry[key2].text = "Repaired Text " .. key2;
				end
				if(BC_Saved.cries[key].cry[key2].condition == nil) then
					BC_Saved.cries[key].cry[key2].condition = { };
				end
				if(BC_Saved.cries[key].cry[key2].name == nil) then
					BC_Saved.cries[key].cry[key2].name = "Repaired " .. key2;
				end
				if(BC_Saved.cries[key].cry[key2].exclude == nil) then
					BC_Saved.cries[key].cry[key2].exlude = 0;
				end
				if(BC_Saved.cries[key].cry[key2].mode == nil) then
					BC_Saved.cries[key].cry[key2].mode = 1;
				end
				if(BC_Saved.cries[key].cry[key2].language == nil) then
					BC_Saved.cries[key].cry[key2].language = 1;
				end
				if(BC_Saved.cries[key].cry[key2].start == 1) then
					BC_Saved.cries[key].cry[key2].castCode = 2;
				elseif(BC_Saved.cries[key].cry[key2].interrupt == 1) then
					BC_Saved.cries[key].cry[key2].castCode = 3;
				elseif(BC_Saved.cries[key].cry[key2].castCode == nil) then
					BC_Saved.cries[key].cry[key2].castCode = 1;
				end
				BC_Saved.cries[key].cry[key2].start = nil;
				BC_Saved.cries[key].cry[key2].interrupt = nil;
				if(type(BC_Saved.cries[key].cry[key2].frequency) ~= "number") then
					BC_Saved.cries[key].cry[key2].frequency = 1;
				elseif(BC_Saved.cries[key].cry[key2].frequency > 10) then
					BC_Saved.cries[key].cry[key2].frequency = 1;
				end
			end
		end
	end
end

--===================================================================================================================--

-- Database Population Functions

--===================================================================================================================--

function BC_PopulateSpellbook()
	BC_Debug("BC_PopulateSpellbook()",2);
	BC_Spellbook = {};
	local spellCount = 1;
	local newTab = { };
	for i=1, MAX_SKILLLINE_TABS do
		name, _, _, numSpells = GetSpellTabInfo(i);
		if not name then
			do break end
		end
		local spellsAdded = 1;
		for j=1, numSpells do
			actionName, spellRank = GetSpellName(spellCount, BOOKTYPE_SPELL);
			if not actionName then
				do break end
			end
			if(IsPassiveSpell(spellCount, BOOKTYPE_SPELL)==1) then
			else
				local found = false;
				actionName = BC_ClearQualifiers(actionName);
				for k=1, spellsAdded do
					if(newTab[k] == actionName) then
						found = true;
					end
				end
				if( not found) then
					newTab[spellsAdded] = actionName;
					spellsAdded = spellsAdded + 1;
				end
				if(BC_Saved.cries[actionName] == nil) then
					BC_Saved.cries[actionName] = { frequency = 0, cooldown = 0, cry = {}, };
				else
					if(not BC_Saved.cries[actionName].cry) then
						BC_Saved.cries[actionName].cry = {};
					end
				end
			end
			spellCount = spellCount + 1;
		end
		BC_Spellbook[name] = newTab;
		newTab = { };
	end
	if(not BC_Saved.actions) then
		BC_Saved.actions = {};
	end
	for i=1, table.getn(BC_Saved.actions) do
		newTab[i] = BC_Saved.actions[i];
	end
	BC_Spellbook[BC_VARS_CATEGORYCUSTOM] = newTab;
	newTab = {};
	i = 1;
	if(not BC_Saved.events) then
		BC_Saved.events = {};
	end
	for key in pairs(BC_Saved.events) do
		newTab[i] = key;
		i = i + 1;
	end
	BC_Spellbook[BC_VARS_CATEGORYEVENTS] = newTab;
	BC_Initialized.spellbook = true;
end

function BC_PopulatePetbook()
	BC_Debug("BC_PopulatePetbook()",2);
	if(HasPetUI()) then
		local newTab = { };
		spellsCovered = 63;
		spellsAdded = 1;
		for i= 1, NUM_PET_ACTION_SLOTS, 1 do
			local actionName, _, _, _, _, _, _ = GetPetActionInfo(i);
			if(actionName) then
				if(actionName=="PET_ACTION_ATTACK") then
					actionName = "Pet Attack";
					spellsCovered = spellsCovered - 1;
					BC_Debug("BC_PopulatePetbook found " .. actionName .. " and Spells Covered = " .. spellsCovered, 3);
				elseif(actionName=="PET_ACTION_FOLLOW") then
					actionName = "Pet Follow";
					spellsCovered = spellsCovered - 2;
					BC_Debug("BC_PopulatePetbook found " .. actionName .. " and Spells Covered = " .. spellsCovered, 3);
				elseif(actionName=="PET_ACTION_WAIT") then
					actionName = "Pet Stay";
					spellsCovered = spellsCovered - 4;
					BC_Debug("BC_PopulatePetbook found " .. actionName .. " and Spells Covered = " .. spellsCovered, 3);
				elseif(actionName=="PET_MODE_AGGRESSIVE") then
					actionName = "Pet Aggressive";
					spellsCovered = spellsCovered - 8;
					BC_Debug("BC_PopulatePetbook found " .. actionName .. " and Spells Covered = " .. spellsCovered, 3);
				elseif(actionName=="PET_MODE_DEFENSIVE") then
					actionName = "Pet Defensive";
					spellsCovered = spellsCovered - 16;
					BC_Debug("BC_PopulatePetbook found " .. actionName .. " and Spells Covered = " .. spellsCovered, 3);
				elseif(actionName=="PET_MODE_PASSIVE") then
					actionName = "Pet Passive";
					spellsCovered = spellsCovered - 32;
					BC_Debug("BC_PopulatePetbook found " .. actionName .. " and Spells Covered = " .. spellsCovered, 3);
				else
					actionName = "Pet " .. actionName;
				end
				if(actionName) then
					newTab[spellsAdded] = actionName;
					spellsAdded = spellsAdded + 1;
					if(BC_Saved.cries[actionName] == nil) then
						BC_Saved.cries[actionName] = { frequency = 0, cooldown = 0, cry = {}, };
						BC_Saved.cries[actionName].petAction = true;
					else
						BC_Saved.cries[actionName].petAction = true;
					end
				end
			end
		end
		if(spellsCovered >= 32) then
			actionName = "Pet Passive";
			newTab[spellsAdded] = actionName;
			spellsAdded = spellsAdded + 1;
			spellsCovered = spellsCovered - 32;
		end
		if(spellsCovered >= 16) then
			actionName = "Pet Defensive";
			newTab[spellsAdded] = actionName;
			spellsAdded = spellsAdded + 1;
			spellsCovered = spellsCovered - 16;
		end
		if(spellsCovered >= 8) then
			actionName = "Pet Aggressive";
			newTab[spellsAdded] = actionName;
			spellsAdded = spellsAdded + 1;
			spellsCovered = spellsCovered - 8;
		end
		if(spellsCovered >= 4) then
			actionName = "Pet Stay";
			newTab[spellsAdded] = actionName;
			spellsAdded = spellsAdded + 1;
			spellsCovered = spellsCovered - 4;
		end
		if(spellsCovered >= 2) then
			actionName = "Pet Follow";
			newTab[spellsAdded] = actionName;
			spellsAdded = spellsAdded + 1;
			spellsCovered = spellsCovered - 2;
		end
		if(spellsCovered == 1) then
			actionName = "Pet Attack";
			newTab[spellsAdded] = actionName;
		end		
		BC_Spellbook[BC_VARS_CATEGORYPET] = newTab;
	end
	BC_Initialized.petbook = true;
end

function BC_PopulateLanguages()
	BC_Debug("BC_PopulateLanguages()",2);
	if(not BC_Initialized.languages) then
		BC_Languages = { };
		if(GetNumLaguages()) then
			for i=1, GetNumLaguages() do
				BC_Languages[i] = GetLanguageByIndex(i);
				BC_Debug("Adding " .. BC_Languages[i] .. " as " .. i, 3);
				if(BC_Languages[i] == "Common" or BC_Languages[i] == "Orcish") then
					BC_Defaults.language = i;
				end
			end
			BC_Initialized.languages = true;
		else
			BC_Initialized.languages = false;
		end
	end
end

--===================================================================================================================--

-- Misc Functions

--===================================================================================================================--

function BC_SlashHandler(msg)
	BC_Debug("BC_SlashHandler(" .. msg .. ")", 2);
	if(msg==nil or msg == "") then
		if(BC_MainFrame:IsVisible()) then
			BC_MainFrame:Hide();
		else
			BC_MainFrame:Show();
		end
	elseif(strsub(msg,1,5) == "debug") then
		BC_ToggleDebug();
	elseif(strsub(msg,1,5) == "reset") then
		BC_ResetSpellbook();
	elseif(strsub(msg,1,4) == "vars") then
		BC_DisplayVariables();
	elseif(strsub(msg,1,4) == "echo") then
		BC_ToggleEcho();
	elseif(strsub(msg,1,5) == "emote") then
		BC_DisplayEmotes();
	elseif(strsub(msg,1,4) == "cry ") then
		actionName = strsub(msg,5);
		BC_Debug("|" .. actionName .. "|");
		BC_ActionManagement(actionName, 0);
		BC_ExecuteCry(BC_Loaded.cry);
	else
		BC_DisplayHelp();
	end
end

function BC_ToggleEcho()
	if(BC_Vars.echo==true) then
		BC_Message(gsub(BC_TEXT_ECHOTOGGLE,"XONOFFX", BC_TEXT_OFF));
		BC_Vars.echo=false;
	else
		BC_Message(gsub(BC_TEXT_ECHOTOGGLE,"XONOFFX", BC_TEXT_ON));
		BC_Vars.echo=true;
	end
end

function BC_ToggleDebug()
	if(BC_Vars.debug==true) then
		BC_Debug("Debug mode off.");
		BC_Vars.debug=false;
	else
		BC_Vars.debug=true;
		BC_Debug("Debug mode on.");
	end
end

function BC_DisplayVariables()
	for i=1, table.getn(BC_TEXT_HELPVARS) do
		BC_Message("|CFF40F040" .. BC_CONDITION_VARIABLES[i] .. "|r - " .. BC_TEXT_HELPVARS[i]);
	end
end

function BC_DisplayHelp()
	for i=1, table.getn(BC_TEXT_HELP) do
		BC_Message(BC_TEXT_HELP[i]);
	end
end

function BC_ClearQualifiers(actionName)
--	BC_Debug("BC_ClearQualifiers(" .. actionName .. ")", 2);
	if(not actionName) then return; end
	for i=1, table.getn(BC_VARS_QUALIFIERS) do
		if(strfind(actionName," %(" .. BC_VARS_QUALIFIERS[i] .. "%)")) then
			actionName = gsub(actionName," %(" .. BC_VARS_QUALIFIERS[i] .. "%)","");
		end
	end
	return actionName;
end

function BC_CastPetAction(index)
	BC_Debug("BC_CastPetAction(" .. index .. ")", 2);
	local actionName, _, _, _, _, _, _ = GetPetActionInfo(index);
	local petName = "Pet ";
	if(actionName) then
		if(actionName=="PET_ACTION_ATTACK") then
			actionName = "Attack";
		end
		if(actionName=="PET_ACTION_FOLLOW") then
			actionName = "Follow";
		end
		if(actionName=="PET_ACTION_WAIT") then
			actionName = "Stay";
		end
		if(actionName=="PET_MODE_PASSIVE") then
			actionName = "Passive";
		end
		if(actionName=="PET_MODE_AGGRESSIVE") then
			actionName = "Aggressive";
		end
		if(actionName=="PET_MODE_DEFENSIVE") then
			actionName = "Defensive";
		end
		actionName = petName .. actionName;
	
		BC_Debug(actionName, 3);
		BC_ActionManagement(actionName, 0);
		BC_ExecuteCry(BC_Loaded.cry);
	end	
end
